#-----------------------------------------------------------------------------
# Version     : 2.4 (3/5/2017)
# Name        : jimhami42_rack_and_pinion.rb
# Description : This plugin creates a involute tooth gear (with or without a hole for shaft and key)
#             : and/or a linear (rack) gear using the same applicable parameters (with optional axis)
# Parameters  : The number of teeth, pressure angle, pitch radius, shaft radius, keyway width and depth.
# Menu Item   : Rack and Pinion
# Context Menu: NONE
# Usage       : N/A
#----------------------------------------------------------------------------- 
# Date        : 9/2/2006
# Author      : Doug Herrmann
# Notes       : The original involute gear code by Doug Hermann was the original core code but has been
#             : substantially reworked. However, credit is included to recognize his contributions.
#-----------------------------------------------------------------------------
# Date        : 4/7/2015
# Author      : Jim Hamilton
#             : Modified last lines to add to the "Plugins" menu option and to work with SketchUp 2015.
#-----------------------------------------------------------------------------
# Date        : 4/18/2015
# Author      : Jim Hamilton
# Notes       : Re-organized Doug Herrmann's code and made some changes here and there. All original
#             : code and nomenclature has been retained where applicable with "DH" (or "dh") sprinkled
#             : in various places. This is by no means an attempt to pass off Doug Herrmann's code as
#             : my own but an attempt to make it more useable with today's releases of SketchUp.
#             : The current version only replicates the basic gear function and does not provide the
#             : handy keyway slot. With apologies to Doug Herrmann and mucho kudos to his work!
#             :
#             : Modified fillet routine to create more points and better end tangency
#-----------------------------------------------------------------------------
# Date        : 4/20/2015
# Author      : Jim Hamilton
# Notes       : Added in the hole and/or keyway slot - if the hole diameter is 0, no hole is created
#             : If the hole diameter is not zero and the keyway width is 0, then only a hole is created
#             : If the hole diameter and keyway width are both not 0, then the keyway is created
#             : Use common sense for these numbers as no error checking is performed on the input
#-----------------------------------------------------------------------------
# Date        : 8/24/2015
# Author      : Jim Hamilton
# Notes       : Modified UI input to work with any units (cannot use "," as a decimal separator)
#-----------------------------------------------------------------------------
# Date        : 8/25/2015
# Author      : Jim Hamilton
# Notes       : Modified UI input to cast dict input to length on input
#-----------------------------------------------------------------------------
# Date        : 11/4/2015
# Author      : Jim Hamilton
# Notes       : Restructured plugin to follow SketchUp class template
#-----------------------------------------------------------------------------
# Date        : 11/5/2015
# Author      : Jim Hamilton
# Notes       : Added pinion/rack/both options
#-----------------------------------------------------------------------------
# Date        : 11/6/2015
# Author      : Jim Hamilton
# Notes       : Added thickness to parts
#-----------------------------------------------------------------------------
# Date        : 11/7/2015
# Author      : Jim Hamilton
# Notes       : Modified tip height calculation for the rack teeth to minimize interference
#-----------------------------------------------------------------------------
# Date        : 12/24/2015
# Author      : Jim Hamilton
# Notes       : Modified dictionary inputs using TIG's suggestion to work with european decimals
#-----------------------------------------------------------------------------
# Date        : 8/6/2016
# Author      : Jim Hamilton
# Notes       : Added optional axis display using construction lines and points
#-----------------------------------------------------------------------------
# Date        : 3/5/2017
# Author      : Jim Hamilton
# Notes       : Added plus and minus tip and root values
#=============================================================================
require 'sketchup.rb'
#=============================================================================
module Jimhami42
  module RackAndPinion
# MAIN BODY -----------------------------------------------------------------------------
    class << self
      def jimhami42_rack_and_pinion_involute_point(ang,rad_base)
        x1 = rad_base * Math.cos(ang)
        y1 = rad_base * Math.sin(ang)
        s = ang * rad_base
        x = x1 + s * Math.sin(ang)
        y = y1 - s * Math.cos(ang)
        [x,y]
      end
#=============================================================================
      def jimhami42_rack_and_pinion_fillet_points(x0,y0,rad_fillet)
        points = []
        ang = Math::PI 
#        7.times do #JRH one less point needed here (fencepost error)
        6.times do #JRH one less point needed here (fencepost error)
          x = x0 + rad_fillet * Math.cos(ang)
          y = y0 + rad_fillet * Math.sin(ang)
          points.push([x,y,0])
          ang = ang - Math::PI / 12.0
        end
        points
      end
#=============================================================================
      def jimhami42_rack_and_pinion_input()
        model = Sketchup.active_model
        model.start_operation "Rack and Pinion"
        m_entities = model.active_entities
        dict = model.attribute_dictionaries["Jimhami42_Rack_and_Pinion_Parameters"]
        if !dict
          dict = model.attribute_dictionary("Jimhami42_Rack_and_Pinion_Parameters",true)
          jimhami42_rack_and_pinion_init_dictionary(dict)
        end
        prompts = ["Number of Teeth: ","Pressure Angle: ","Pitch Radius: ","Step Angle: ","Hole Dia: ","Keyway Width: ","Type: ","Thickness: ","Add axis: ","Tip(+/-): ","Base(+/-): "]
        defaults = [dict['teeth'],dict['p_ang'],dict['p_rad'].to_l,dict['s_ang'],dict['h_dia'].to_l,dict['k_width'].to_l,dict['type'],dict['thick'].to_l,dict['axis'],dict['plus'],dict['minus']]
        list = ["","","","","","","Pinion|Rack|Both","","No|Yes","",""]
        input = UI.inputbox prompts, defaults, list, "Enter Gear Parameters"
        if(input)
          teeth = input[0].to_i
          p_ang = input[1].to_f * Math::PI / 180.0
          p_rad = input[2].to_l
          s_ang = input[3].to_f * Math::PI / 180.0
          h_dia = input[4].to_l
          k_width = input[5].to_l
          type = input[6]
          thick = input[7].to_l
          axis = input[8]
          plus = input[9].to_f
          minus = input[10].to_f
          dict['teeth'] = input[0]
          dict['p_ang'] = input[1]
          dict['p_rad'] = input[2]
          dict['s_ang'] = input[3]
          dict['h_dia'] = input[4]
          dict['k_width'] = input[5]
          dict['type'] = input[6]
          dict['thick'] = input[7]
          dict['axis'] = input[8]
          dict['plus'] = input[9]
          dict['minus'] = input[10]
          if(type == 'Pinion' || type == 'Both')
#------ CREATE PINION --------------------------------------
            group = m_entities.add_group
            entities = group.entities
            diameter = 2.0 * p_rad
            rad_inside = p_rad - diameter / teeth #DH - 1/diametral pitch
            rad_outside = p_rad + diameter / teeth + plus #DH + 1/diametral pitch
            rad_root = p_rad - 1.157 * diameter / teeth + minus #DH - 1.157/diametral pitch
            rad_base = p_rad * Math.cos(p_ang)
            rad_fillet = rad_inside - rad_root
            points = []
           #DH Fillet points on right side of tooth
            if rad_inside <= rad_base
              points = jimhami42_rack_and_pinion_fillet_points(rad_inside,-rad_fillet,rad_fillet)
              ang = 0.0
            else #DH calc angle and coordinate involute intersects inside radius
              s = Math.sqrt(rad_inside**2 - rad_base**2)
              ang = s / rad_base
              x0,y0 = jimhami42_rack_and_pinion_involute_point(ang,rad_base)
              points = jimhami42_rack_and_pinion_fillet_points(x0,y0 - rad_fillet,rad_fillet)
            end
           #DH calc value of parameter where Involute curve intersects outside radius
            s_max = Math.sqrt(rad_outside**2 - rad_base**2)
            ang_max = s_max / rad_base
           #DH calc Involute curve points along tooth's right side
            ang += s_ang #JRH (radians) added as separate input
            while ang < ang_max
              x,y = jimhami42_rack_and_pinion_involute_point(ang,rad_base)
              points.push([x,y,0])
              ang += s_ang #JRH (radians) added as separate input
            end
           #DH Compute last point where involute intersects the outside circle
            x,y = jimhami42_rack_and_pinion_involute_point(ang_max,rad_base)
            points.push([x,y,0])
           #DH Rotate profile CW so that it intersects the x-axis at the Pitch radius
            s = Math.sqrt(p_rad**2 - rad_base**2)
            delta = (s / rad_base) - p_ang
            t0 = Geom::Transformation.rotation([0,0,0],Z_AXIS,-delta)
            points.collect! {|pnt| t0 * pnt}
           #DH Reflect about the x-axis
            t1 = Geom::Transformation.rotation([0,0,0],X_AXIS,Math::PI)
           #DH  Rotate CW a tooth-space
            t2 = Geom::Transformation.rotation([0,0,0],Z_AXIS,-Math::PI / teeth)
           #DH The left profile of the adjacent tooth
            t = t2 * t1
            reflect = points.collect {|pnt| t * pnt}
           #DH Left profile, space, right profile
            points = reflect.reverse! + points
           #DH Center tooth along x-axis
            t3 = Geom::Transformation.rotation([0,0,0],Z_AXIS,-Math::PI / (2.0 * teeth))
            points.collect! {|pnt| t3 * pnt}
           #DH vertices for edge
            p1 = points.last
            p2 = [p1.x,-p1.y,0]
           #DH Now add curves and edges outlining the gear
            edges = []
            0.upto(teeth - 1) do |n|
              tr = Geom::Transformation.rotation([0,0,0],Z_AXIS,2.0 * Math::PI * n / teeth)
             #DH add curve
              edges += entities.add_curve points.collect {|pnt| tr * pnt}
              entities.add_curve points.collect {|pnt| tr * pnt}
             #DH add edge at top of tooth
              edges += entities.add_edges tr*p1, tr*p2
              entities.add_edges tr*p1, tr*p2
            end
            if(axis == 'Yes')
              entities.add_cpoint([0,0,0])
            end
#------ CREATE KEYWAY --------------------------------------
            if(h_dia != 0.0)
              h_rad = h_dia / 2.0
              a1 = Math.asin((k_width / 2.0) / h_rad)
              if(teeth < 48)
                hres = 48
              else
                hres = teeth
              end
              if(k_width != 0.0)
                k_depth = 0.75 * k_width
                x = h_rad * Math.cos(a1)
                y = h_rad * Math.sin(a1)
                edges1 = entities.add_edges([x,-y,0],[x + k_depth,-y,0],[x + k_depth,y,0],[x,y,0])
                edges1 += entities.add_arc([0,0,0],[1,0,0],[0,0,1],h_rad,a1,2.0 * Math::PI - a1,hres)
                face1 = entities.add_face edges1
              else
                edges1 = entities.add_circle([0,0,0],[0,0,1],h_rad,hres)
                face1 = entities.add_face(edges1)
              end
              gear = entities.add_face(edges)
              face1.erase!
            else  
              gear = entities.add_face(edges)
            end
            if(thick != 0.0)
              gear.pushpull(-thick,false)
              if(axis == 'Yes')
                entities.add_cpoint([0,0,thick])
                entities.add_cline([0,0,0],[0,0,thick])
              end
            end
          end
          if(type == 'Rack' || type == 'Both')
#------ CREATE RACK ----------------------------------------
            group = m_entities.add_group
            entities = group.entities
            points = []
            dmp = 2.0 * p_rad / teeth
            clr = 1.157 * dmp
            pitch = dmp * Math::PI
            px = p_rad
            py = -(pitch / 4.0)
            dmp *= Math.sin(p_ang) # shortens the tip according to the angle
            for i in 0...teeth
              points.push([px + clr,py - pitch / 2.0 - clr * Math.tan(p_ang) + i.to_f * pitch,0])
              points.push([px - dmp,py - pitch / 2.0 + dmp * Math.tan(p_ang) + i.to_f * pitch,0])
              points.push([px - dmp,py - dmp * Math.tan(p_ang) + i.to_f * pitch,0])
              points.push([px + clr,py + clr * Math.tan(p_ang) + i.to_f * pitch,0])
            end
            points.push([2.0 * p_rad,points.last[1],0])
            points.push([2.0 * p_rad,points.first[1],0])
            points.push([points.first[0],points.first[1],0])
            entities.add_edges(points)
            rack = entities.add_face(points)
            if(thick != 0.0)
#------ ADD HEIGHT ------------------------------------------
              rack.pushpull(-thick,false)
            end
          end
        end
        model.commit_operation
      end
#=============================================================================
      def jimhami42_rack_and_pinion_init_dictionary(dict)
        dict['teeth'] = '12'
        dict['p_ang'] = '20.0'
        dict['p_rad'] = '1"'
        dict['s_ang'] = '2.0'
        dict['h_dia'] = '0"'
        dict['k_width'] = '0"'
        dict['type'] = 'Pinion'
        dict['thick'] = '0"'
        dict['axis'] = 'No'
        dict['plus'] = '0.0'
        dict['minus'] = '0.0'
      end
    end
    unless file_loaded?("jimhami42_rack_and_pinion.rb")
      menu = UI.menu("PlugIns").add_item("Rack and Pinion") { jimhami42_rack_and_pinion_input() }
      file_loaded("jimhami42_rack_and_pinion.rb")
    end
  end
end
#=============================================================================
